/*****************************************************************************\
 *                        ANALYSIS PERFORMANCE TOOLS                         *
 *                             ClusteringSuite                               *
 *   Infrastructure and tools to apply clustering analysis to Paraver and    *
 *                              Dimemas traces                               *
 *                                                                           *
 *****************************************************************************
 *     ___     This library is free software; you can redistribute it and/or *
 *    /  __         modify it under the terms of the GNU LGPL as published   *
 *   /  /  _____    by the Free Software Foundation; either version 2.1      *
 *  /  /  /     \   of the License, or (at your option) any later version.   *
 * (  (  ( B S C )                                                           *
 *  \  \  \_____/   This library is distributed in hope that it will be      *
 *   \  \__         useful but WITHOUT ANY WARRANTY; without even the        *
 *    \___          implied warranty of MERCHANTABILITY or FITNESS FOR A     *
 *                  PARTICULAR PURPOSE. See the GNU LGPL for more details.   *
 *                                                                           *
 * You should have received a copy of the GNU Lesser General Public License  *
 * along with this library; if not, write to the Free Software Foundation,   *
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA          *
 * The GNU LEsser General Public License is contained in the file COPYING.   *
 *                                 ---------                                 *
 *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
\*****************************************************************************/

/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\

  $Id::                                           $:  Id
  $Rev::                                          $:  Revision of last commit
  $Author::                                       $:  Author of last commit
  $Date::                                         $:  Date of last commit

\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */


#include <types.h>
#include <libTraceClustering.hpp>

#include <FileNameManipulator.hpp>
using cepba_tools::FileNameManipulator;

#include <string>
using std::string;

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <sstream>
using std::stringstream;

string ClusteringDefinitionXML;    /* Clustering definition XML file name */
bool   ClusteringDefinitionRead = false;

string InputTraceName;             /* Input trace name */
string InputSemanticCSV;           /* Special case to integrate with Paraver */
string InputTraceNamePrefix;
bool   InputTraceNameRead   = false;
bool   InputSemanticCSVRead = false;

string OutputDataFileNamePrefix;   /* The filename of the input trace without
                                      extension */

bool   ApplyCPIStack             = false;

string EigenMatrixFileName;

bool  ExtractNormalizedData      = false;

bool  ChangeBase                 = false;

bool              UseParaverEventParsing = false;
bool              ConsecutiveEvts        = false;
set<unsigned int> EventsToParse;

#define HELP \
"%s v%s (%s %s)\n"\
"(c) BSC Tools - Barcelona Supercomputing Center\n"\
"\n"\
"Usage:\n"\
"  %s -d <clustering_def.xml> [-x] -i <input_trace> -o <output_data>\n"\
"\n"\
"  -a                         Information about the tool\n"\
"\n"\
"  -h                         This help\n"\
"\n"\
"  -s                         Silent mode (No progress messages)\n"\
"\n"\
"  -e Type1,Type2,...         When using an input Paraver trace, use this\n"\
"                             event types to determine the regions treated as\n"\
"                             bursts\n"\
"\n"\
"  -d <clustering_def_xml>    XML containing the parameters definition\n"\
"\n"\
"  -c                         Extract CPI stack data for PPC970 native counters\n"\
"\n"\
"  -x                         Generate plot for *normalized* data from input\n"\
"                             file (Default: *raw* data plot)\n"\
"\n"\
"  -m <eigen_matrix_file>     CSV file containing an eigenvectors matrix to\n"\
"                             transform the original space\n"\
"\n"\
"\n"\
"  -i <input_file | semantic_timeline.csv,tracefile.prv >\n"\
"\n"\
"                              Input CSV | Dimemas trace | Paraver trace |\n"\
"                              Combination of semantic timeline csv file\n"\
"                              generated by Paraver and the corresponding trace\n"\
"\n"\


#define ABOUT \
"\n"\
"%s v%s (%s %s)\n"\
"(c) BSC Tools - Barcelona Supercomputing Center\n"\
"Data extraction utility for Paraver/Dimemas traces\n"

void GetEventParsingParameters(char* EventParsingArgs);

void PrintUsage(char* ApplicationName)
{
  cout << "Usage: " << ApplicationName << " -d <clustering_def.xml>";
  cout << "[-x] [-m <eigen_matrix_file>] -i <input_trace>" << endl;
}

void ReadArgs(int argc, char *argv[])
{
  INT32 j = 1;
  INT32 ParametersRequired = 1;

  string InputFiles;
  size_t Comma;

  if (argc == 1 ||
      argc == 2 &&
      ((strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "--help") == 0)))
  {
    fprintf(stdout, HELP, argv[0], VERSION, __DATE__, __TIME__, argv[0]);
    exit(EXIT_SUCCESS);
  }

  if (argc == 1 ||
      argc == 2 &&
      ((strcmp(argv[1], "-a") == 0)))
  {
    fprintf(stdout, ABOUT, argv[0], VERSION, __DATE__, __TIME__);
    exit(EXIT_SUCCESS);
  }

  /*
  if (argc != 6 && argc != 7 && argc != 8)
  {
    PrintUsage(argv[0]);
    exit(EXIT_FAILURE);
  }
  */

  if (argv[1][0] == '-')
  {
    for (j = 1; (j < argc - 1) && (argv[j][0] == '-'); j++)
    {
      switch (argv[j][1])
      {
        case 'd':
          j++;
          ClusteringDefinitionXML  = argv[j];
          ClusteringDefinitionRead = true;
          break;
        case 'i':
          j++;
          InputFiles = argv[j];
          Comma      = InputFiles.find(',');

          if (Comma != string::npos)
          {
            InputSemanticCSV      = InputFiles.substr(0, Comma);
            InputTraceName        = InputFiles.substr(Comma+1);
            InputSemanticCSVRead  = true;

            /* DEBUG */
            cout << "InputSemanticCSV = " << InputSemanticCSV << endl;
            cout << "InputTraceName = " << InputTraceName << endl;
          }
          else
          {
            InputTraceName     = InputFiles;
          }

          InputTraceNameRead = true;
          break;
        case 's':
          // State::SetSilentMode(false);
          break;
        case 'x':
          ExtractNormalizedData = false;
          break;
        case 'm':
          ChangeBase = true;
          j++;
          EigenMatrixFileName = argv[j];
          break;
        /*
        case 'o':
          j++;
          OutputFileName     = argv[j];
          OutputFileNameRead = true;
          break;
        */
        case 'e':
          UseParaverEventParsing = true;

          if (argv[j][2] == 'c')
          {
            ConsecutiveEvts = true;
          }

          j++;
          GetEventParsingParameters(argv[j]);
          break;

        /*
        case 'g':
          GenerateGNUPlotScript = true;
          break;
        */
        default:
          cerr << "Invalid parameter " << argv[j][1] << endl;
          PrintUsage(argv[0]);
          exit(EXIT_FAILURE);
          break;
      }
    }
  }

  if (!ClusteringDefinitionRead)
  {
    cerr << "Definition XML file missing ( \'-d\' parameter)" << endl;
    exit (EXIT_FAILURE);
  }

  if (!InputTraceNameRead)
  {
    cerr << "Input trace missing ( \'-i\' parameter)" << endl;
    exit (EXIT_FAILURE);
  }

  /*
  if (!OutputFileNameRead)
  {
    cerr << "Output data file file missing ( \'-o\' parameter)" << endl;
    exit (EXIT_FAILURE);
  }
  */

  return;
}

void GetEventParsingParameters(char* EventParsingArgs)
{
  char* err;

  string       ArgsString (EventParsingArgs);
  stringstream ArgsStream (ArgsString);
  string       Buffer;

  set<unsigned int>::iterator EventsIterator;

  while(std::getline(ArgsStream, Buffer, ','))
  {
    unsigned int CurrentType;

    CurrentType = strtoul(Buffer.c_str(), &err, 0);
    if (*err)
    {
      cerr << "Error on event parsing parameters (\'-e\'): Incorrect type ";
      cerr << "(" << Buffer << ")" << endl;
      exit (EXIT_FAILURE);
    }
    else
    {
      EventsToParse.insert(CurrentType);
    }
  }

  /* DEBUG */
  cout << "Events to parse: ";
  for (EventsIterator  = EventsToParse.begin();
       EventsIterator != EventsToParse.end();
       ++EventsIterator)
  {
    cout << (*EventsIterator) << " ";
  }
  cout << endl;
}

void GenerateOutputFileNamePrefix()
{
  string OutputFileExtension;

  OutputFileExtension = FileNameManipulator::GetExtension(InputTraceName);

  if (OutputFileExtension.compare("") == 0)
  {
    cerr << "Unable to determine input file type. Please use .prv/.trf extensions" << endl;
    exit (EXIT_FAILURE);
  }

  if (OutputFileExtension.compare("prv") == 0 ||
      OutputFileExtension.compare("trf") == 0)
  {
    FileNameManipulator NameManipulator(InputTraceName, OutputFileExtension);

    OutputDataFileNamePrefix = NameManipulator.GetChoppedFileName();

    return;
  }
  else
  {
    cerr << "Unknown input file type. Please use .prv/.trf to choose the input file type" << endl;
    exit(EXIT_FAILURE);
  }

  return;
}

int main(int argc, char *argv[])
{
  libTraceClustering Clustering = libTraceClustering(true);

  ReadArgs(argc, argv);

  GenerateOutputFileNamePrefix();

  if (!Clustering.InitTraceClustering(ClusteringDefinitionXML,
                                      OutputDataFileNamePrefix+".pcf",
                                      false, // IMPORTANTE: NEED TO BE CHECKED
                                      false, // IMPORTANTE: NEED TO BE CHECKED
                                      PLOTS))
  {
    cerr << "Error seting up clustering library: " << Clustering.GetErrorMessage() << endl;
    exit (EXIT_FAILURE);
  }

  if (UseParaverEventParsing)
  {
    if (!Clustering.ExtractData(InputTraceName, false, 0, EventsToParse)) // false -> No Samplig , 0 -> Max Samples
    {
      cerr << "Error extracting data: " << Clustering.GetErrorMessage() << endl;
      exit (EXIT_FAILURE);
    }
  }
  else
  {
    if (!Clustering.ExtractData(InputTraceName,
                                EventsToParse,
                                ConsecutiveEvts,
                                InputSemanticCSV))
    {
      cerr << "Error extracting data: " << Clustering.GetErrorMessage() << endl;
      exit (EXIT_FAILURE);
    }
  }

  if (!Clustering.FlushData(OutputDataFileNamePrefix))
  {
    cerr << "Error writing data points: " << Clustering.GetErrorMessage() << endl;
    exit (EXIT_FAILURE);
  }

  if (!Clustering.PrintPlotScripts(OutputDataFileNamePrefix))
  {
    cerr << "Error printing plot scripts: " << Clustering.GetErrorMessage() << endl;
    exit (EXIT_FAILURE);
  }

  exit (EXIT_SUCCESS);
}
